from pyspark.sql import DataFrame
import pyspark
import os
from pyspark.sql import DataFrame


def select_columns_from_pyspark_dataframe( dataframe: DataFrame, user_columns: list) -> DataFrame:    
    if len(user_columns) > 0:
        if not isinstance(user_columns, list):
            raise Exception("columns needs to be a valid python list(array), other data type was parsed")
        dataset_columns = list(dataset.columns)
        if not all(elem in dataset_columns  for elem in user_columns): 
            missing_elements = list(set(user_columns) - set(dataset_columns))
            raise Exception("Following columns not in the table requested : " + str(missing_elements)) 
        else:
            return dataset.select(*user_columns)
    else:
        return dataframe


#connection_string = "jdbc:postgresql://localhost:5432/" +  DB_NAME
def get_postgres_table_data(table_name):
    dataset = pyspark_sql_context.read.format("jdbc") \
        .option("url", connection_string) \
        .option("dbtable", table_name) \
        .option("user", DB_USER) \
        .option("password", DB_PASSWORD) \
        .option("encrypt",True) \
        .option("trustServerCertificate",True) \
        .option("driver", "org.postgresql.Driver") \
        .load()
    return dataset


class Postgres():
    def __init__(self, db_name:str, username:str, password:str, spark_sql_context,  url:str):
        self.url = url
        self.username = username
        self.database = db_name
        self.password = password
        self.spark_sql_context = spark_sql_context
        self.connection_string = "jdbc:postgresql://" + self.url + "/" + self.database
        print(self.connection_string)
        
    
    def get_table_dataframe(self, source_table:str ) -> DataFrame:
        dataset = self.spark_sql_context.read.format("jdbc") \
            .option("url", self.connection_string) \
            .option("dbtable", source_table) \
            .option("user", self.username) \
            .option("password", self.password) \
            .option("encrypt",True) \
            .option("trustServerCertificate",True) \
            .option("driver", "org.postgresql.Driver") \
            .load()
        return dataset
    
    
    def append_dataframe_to_table(self, dataframe: DataFrame, destination_table: str):
        self.__write_dataframe_to_table(dataframe, destination_table, "append")
        
        
    def overwrite_dataframe_to_table(self, dataframe: DataFrame, destination_table: str):
        self.__write_dataframe_to_table(dataframe, destination_table, "overwrite")
    
    
    def __write_dataframe_to_table(self, dataframe: DataFrame, destination_table: str, write_mode: str):
        dataframe.write.mode(write_mode).format("jdbc") \
            .option("url", self.connection_string) \
            .option("dbtable", destination_table) \
            .option("user", self.username) \
            .option("password", self.password) \
            .option("encrypt",True) \
            .option("trustServerCertificate",True) \
            .option("driver", "org.postgresql.Driver") \
            .save()
        



#import requests
#import json
import pandas as pd
from pyspark.sql.types import *
import os
from pyspark.sql import DataFrame

class PandasToPyspark:
    def __init__(self, pandas_df, sql_context):
        self.pandas_dataframe = pandas_df
        self.pandas_dataframe_columns = list(self.pandas_dataframe.columns)
        self.pandas_dataframe_datatypes = list(self.pandas_dataframe.dtypes)
        self.sql_context = sql_context
        

    def equivalent_type(self, f):
        if f == 'datetime64[ns]':
            return TimestampType()
        elif f == 'int64':
            return LongType()
        elif f == 'int32':
            return IntegerType()
        elif f == 'float64':
            return DoubleType()
        elif f == 'float32':
            return FloatType()
        else:
            return StringType()

    def define_structure(self, string, format_type):
        try:
            typo = self.equivalent_type(self.format_type)
        except:
            typo = StringType()
        return StructField(string, typo)

    def pandas_to_pyspark(self) -> DataFrame:
        columns = self.pandas_dataframe_columns
        types = self.pandas_dataframe_datatypes
        struct_list = []
        for column, typo in zip(columns, types):
            struct_list.append(self.define_structure(column, typo))
        p_schema = StructType(struct_list)
        return self.sql_context.createDataFrame(self.pandas_dataframe, p_schema)



from pyspark.sql.types import StringType, IntegerType, DateType


def asymmetrical_difference(base_collection, secondary_collection):
    difference_set = list(set(base_collection) - set(secondary_collection))
    return difference_set


def get_class_from_type_name(type_name):
    type_class_maps = {"string": StringType(), "int": IntegerType(), "bigint": IntegerType(), "date": DateType()}
    return type_class_maps[type_name]



from pyspark.sql import SparkSession
import findspark

findspark.init()

postgres_driver_file_path = "drivers/postgresql-42.4.0.jar"
sql_driver_file_path = "drivers/mssql-jdbc-10.2.1.jre8.jar"


"""
import os
post_path = os.getcwd() + postgres_driver_file_path
print("************")
print("post :", post_path)
print("************")
"""

def get_spark_postgres_conf():
    return SparkSession \
        .builder \
        .master('local') \
        .appName('mamba-postgres') \
        .config("spark.driver.extraClassPath", postgres_driver_file_path) \
        .config("spark.sql.debug.maxToStringFields", 255) \
        .config("spark.ui.port", "4060") \
        .config('spark.sql.codegen.wholeStage', 'false') \
        .getOrCreate()


def get_spark_sql_conf():
    return SparkSession \
        .builder \
        .master('local') \
        .appName('mamba') \
        .config("spark.driver.extraClassPath", sql_driver_file_path) \
        .config('spark.sql.codegen.wholeStage', 'false') \
        .enableHiveSupport() \
        .getOrCreate()





from setuptools import setup, find_packages
  
with open("README.md", "r") as fh:
    description = fh.read()
  
setup(
    name="xfsystem",
    version="0.0.2",
    author="Gwamaka Mwamwaja",
    author_email="gwamaka.mwamwaja@ucglobalprograms.org",
    packages=find_packages(),
    description="A package containing reusable functionalities for the mamba-spark project",
    url="https://github.com/GwamakaCharles/ucsf_xfsystem",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


from pyspark.sql import DataFrame
from pyspark.sql.functions import col, array, explode, lit, struct, udf
from pyspark.sql.types import StringType
from typing import Iterable


def rename_multiple_columns(dataframe : DataFrame, column_names: dict ) -> DataFrame:
    for key, value in column_names.items():
        dataframe = dataframe.withColumnRenamed(key, value)
    return dataframe


def transform_dataframe_melt(dataframe, columns_to_melt: Iterable[str], columns_to_transform: Iterable[str],
                             old_column_name: str = "variable", new_column_name: str = "value") -> DataFrame:
    vars_and_vals = array(*(
        struct(lit(c).alias(old_column_name), col(c).alias(new_column_name))
        for c in columns_to_transform))

    df_tmp = dataframe.withColumn("_vars_and_vals", explode(vars_and_vals))

    cols = columns_to_melt + [col("_vars_and_vals")[x].alias(x) for x in [old_column_name, new_column_name]]
    return df_tmp.select(*cols)


def replace_value(column : str, old_value, new_value) -> DataFrame:
    return when(column != old_value, column).otherwise(lit(new_value))


def replace_column_value(dataframe:DataFrame, column : str, old_value, new_value ) -> DataFrame:
    dataframe = dataframe.withColumn(column, replace_value(col(column), old_value, new_value))
    return dataframe


import requests
import json
from io import BytesIO
import pandas as pd
#from pyspark.sql.types import *
import os
#from pyspark.sql import DataFrame
#mport mestandardstz.util.global_variables as global_val




class Odk: #Odl_central_api/lib
    def __init__(self, odk_central_url: str):
        self.url = odk_central_url

    def get_access_token(self, email:str, password:str) -> str:
        credentials = json.dumps({"email": email, "password": password})
        login_url = self.url + "sessions"
        header = {'Content-Type': 'application/json'}
        login = requests.post(login_url, data=credentials, headers= header).json()
        if 'token' not in login:
            raise ConnectionRefusedError('Could not login and connect to odk server')
        access_token = login['token']
        return access_token
    
    
    def get_form_submission_dataframe(self, form_url:str, access_token:str) -> pd.DataFrame:
        data_url = self.url + form_url
        headers_ = {"Content-Type": "application/json", "Authorization": "Bearer " + access_token}
        submissions = requests.get(data_url, headers=headers_)
        csv_content_bytes = submissions.content
        csv_data = BytesIO(csv_content_bytes)
        data = pd.read_csv(csv_data)
        return data


from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
import calendar
from pyspark.sql.types import DateType
from pyspark.sql import DataFrame
from pyspark.sql.functions import col, udf


def transform_string_column_to_date(dataframe : DataFrame, column_name: str, current_date_format:str = "%Y-%m-%d") -> DataFrame:
    date_convertor =  udf (lambda x: datetime.strptime(x, current_date_format), DateType())
    dataframe = dataframe.withColumn('tmp_column_name', date_convertor(col(column_name)))
    dataframe = dataframe.drop(col(column_name))
    dataframe = dataframe.withColumnRenamed('tmp_column_name', column_name)
    return dataframe


def get_month_offset(self, calender_type: str = 'c') -> int:
    month_offset = 0
    if calender_type == 'c':
        month_offset = 0
    elif calender_type == 'g':
        month_offset = 6
    elif calender_type == 'p':
        month_offset == 3
    else:
        month_offset = 0
    return month_offset


def get_quarter_period(day_date: datetime, calender_type: str = 'c') -> str:
    month_offset = get_month_offset(calender_type)
    new_date = day_date + relativedelta(months=month_offset)
    year_number = new_date.year
    month_number = new_date.month
    quarter_number = int((month_number - 1) / 3) + 1
    year_name = str(year_number)[-2:]
    quarter_name = 'Q' + str(quarter_number) + 'FY' + year_name
    return quarter_name


def get_semi_annual_period(day_date: datetime, calender_type: str = 'c') -> str:
    month_offset = get_month_offset(calender_type)
    new_date = day_date + relativedelta(months=month_offset)
    year_number = new_date.year
    month_number = new_date.month
    quarter_number = int((month_number - 1) / 6) + 1
    year_name = str(year_number)[-2:]
    semi_annual_name = 'S' + str(quarter_number) + 'FY' + year_name
    return semi_annual_name


def get_annual_period(day_date: datetime, calender_type: str = 'c') -> str:
    month_offset = get_month_offset(calender_type)
    new_date = day_date + relativedelta(months=month_offset)
    year_number = new_date.year
    year_name = str(year_number)[-2:]
    annual_period = 'FY' + year_name
    return annual_period


def get_calendar_year(day_date: datetime, calender_type: str = 'c') -> str:
    year_number = str(day_date.year)
    return year_number


def get_month_from_date(day_date: datetime) -> str:
    year_number = day_date.year
    year_name = str(year_number)[-2:]
    month_name = calendar.month_abbr[day_date.month]
    month_period = month_name.upper() + year_name
    return month_period


def get_dates(start_date: datetime, end_date: datetime = datetime.now()) -> list:
    week = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday']
    days = []

    date_diff = end_date - start_date
    for i in range(date_diff.days + 1):
        day = start_date + timedelta(days=i)

        day_dct = {
            "date_id": i + 1,
            "date": day.strftime('%Y-%m-%d'),
            "day_name": week[day.weekday()],
            "calendar_year": get_calendar_year(day),
            "semi_annual_period": get_semi_annual_period(day),
            "quarter_period": get_quarter_period(day),
            "month_period": get_month_from_date(day)
        }
        days.append(day_dct)
    return days